@@ -23,6 +23,12 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.logging.Level;
 import java.util.logging.Logger;
+import org.sleuthkit.datamodel.TskData.FileKnown;
+import org.sleuthkit.datamodel.TskData.TSK_DB_FILES_TYPE_ENUM;
+import org.sleuthkit.datamodel.TskData.TSK_FS_ATTR_TYPE_ENUM;
+import org.sleuthkit.datamodel.TskData.TSK_FS_META_TYPE_ENUM;
+import org.sleuthkit.datamodel.TskData.TSK_FS_NAME_FLAG_ENUM;
+import org.sleuthkit.datamodel.TskData.TSK_FS_NAME_TYPE_ENUM;
 
 /**
  * Layout file object representation of a layout file stored in tsk_files table.
@@ -37,11 +43,13 @@ import java.util.logging.Logger;
  */
 public class LayoutFile extends AbstractFile{
 	
-	//layout ranges associated with this file
-	private List<TskFileRange> ranges;
-	
-	protected LayoutFile(SleuthkitCase db, long obj_id, String name, TskData.TSK_DB_FILES_TYPE_ENUM type) {
-		super(db, obj_id, name, type);
+	protected LayoutFile(SleuthkitCase db, long objId, String name, 
+			TSK_DB_FILES_TYPE_ENUM fileType, 
+			TSK_FS_NAME_TYPE_ENUM dirType, TSK_FS_META_TYPE_ENUM metaType, 
+			TSK_FS_NAME_FLAG_ENUM dirFlag, short metaFlags, 
+			long size, String md5Hash, FileKnown knownState, String parentPath) {
+		super(db, objId, TSK_FS_ATTR_TYPE_ENUM.TSK_FS_ATTR_TYPE_DEFAULT, (short)0, name, fileType, 0L, dirType, metaType, dirFlag, metaFlags, size, 0L, 0L, 0L, 0L, (short)0, 0, 0, md5Hash, knownState, parentPath);
+		//this.size = calcSize(); //update calculated size
 	}
 
 	/**
@@ -49,46 +57,24 @@ public class LayoutFile extends AbstractFile{
 	 * @return number of file layout ranges objects associated
 	 */
 	public int getNumParts() {
-		int size = 0;
+		int numParts = 0;
 		try {
-			size = getRanges().size();
+			numParts = getRanges().size();
 		} catch (TskCoreException ex) {
 			Logger.getLogger(LayoutFile.class.getName()).log(Level.INFO, "Error getting layout content ranges for size", ex);
 		}
-		return size;
-	}
-
-	@Override
-	public List<TskFileRange> getRanges() throws TskCoreException {
-		if(ranges == null) {
-            ranges = getSleuthkitCase().getFileRanges(this.getId());
-        }
-        return ranges;
+		return numParts;
 	}
 
+	
 	@Override
 	public List<Content> getChildren() throws TskCoreException {
-		return Collections.<Content>emptyList();
-	}
-	
-	@Override
-	public List<Long> getChildrenIds() throws TskCoreException {
-		return Collections.<Long>emptyList();
+		return getSleuthkitCase().getAbstractFileChildren(this, TskData.TSK_DB_FILES_TYPE_ENUM.DERIVED);
 	}
 
 	@Override
-    public long getSize() {
-        return calcSize();
-    }
-	
-	@Override
-	public boolean isDir(){
-        return false;
-    }
-	
-	@Override
-	public boolean isFile() {
-		return true;
+	public List<Long> getChildrenIds() throws TskCoreException {
+		return getSleuthkitCase().getAbstractFileChildrenIds(this, TskData.TSK_DB_FILES_TYPE_ENUM.DERIVED);
 	}
     
 	/**
@@ -96,25 +82,28 @@ public class LayoutFile extends AbstractFile{
 	 * @return total content size in bytes
 	 */
     private long calcSize() {
-        long size = 0;
+        long calcSize = 0;
         try {
             for (TskFileRange range : getRanges()) {
-                size += range.getByteLen();
+                calcSize += range.getByteLen();
             }
         }catch (TskCoreException ex) {
-			Logger.getLogger(LayoutFile.class.getName()).log(Level.INFO, "boo", ex);
+			Logger.getLogger(LayoutFile.class.getName()).log(Level.SEVERE, "Error calculating layout file size from ranges", ex);
         }
-        return size;
+        return calcSize;
     }
-	
 
 	@Override
-    public int read(byte[] buf, long offset, long len) throws TskCoreException {
+	public void close() {
+		//nothing to be closed
+	}
+	
+	@Override
+    protected int readInt(byte[] buf, long offset, long len) throws TskCoreException {
         long offsetInThisLayoutContent = 0; // current offset in this LayoutContent
         int bytesRead = 0; // Bytes read so far
-        Iterator<TskFileRange> it = getRanges().iterator();
-        while(it.hasNext()) {
-            TskFileRange range = it.next();
+		
+        for (TskFileRange range : getRanges()) {
             if (bytesRead < len) { // we haven't read enough yet
                 if (offset < offsetInThisLayoutContent + range.getByteLen()) { // if we are in a range object we want to read from
                     long offsetInRange = 0; // how far into the current range object to start reading
@@ -137,15 +126,6 @@ public class LayoutFile extends AbstractFile{
         return bytesRead;
     }
 	
-	/**
-	 * Convert an internal offset to an image offset
-	 * @param layoutOffset the offset in this layout file
-	 * @return the corresponding offset in the image
-	 */
-	public long convertToImgOffset(long layoutOffset) {
-		throw new UnsupportedOperationException("Not supported yet!");
-	}
-    
 	/*
 	 * Read bytes from an image into a buffer, starting at given position in buffer
 	 * 
@@ -179,14 +159,14 @@ public class LayoutFile extends AbstractFile{
 	public Image getImage() throws TskCoreException{
 		return getParent().getImage();
 	}
-	
-	@Override
-	public boolean isVirtual() {
-		return true;
-	}
+
 	
 	@Override
 	public boolean isRoot() {
 		return false;
 	}
+	@Override
+	public String toString(boolean preserveState){
+		return super.toString(preserveState) + "LayoutFile [\t" + "]\t";
+	}
 }
