@@ -130,7 +130,7 @@ class ResultSetHelper {
 				TSK_FS_ATTR_TYPE_ENUM.valueOf(rs.getShort("attr_type")),
 				rs.getShort("attr_id"), rs.getString("name"), rs.getLong("meta_addr"),
 				TSK_FS_NAME_TYPE_ENUM.valueOf(rs.getShort("dir_type")),
-				TSK_FS_META_TYPE_ENUM.ValueOf(rs.getShort("meta_type")),
+				TSK_FS_META_TYPE_ENUM.valueOf(rs.getShort("meta_type")),
 				TSK_FS_NAME_FLAG_ENUM.valueOf(rs.getShort("dir_flags")),
 				rs.getShort("meta_flags"), rs.getLong("size"),
 				rs.getLong("ctime"), rs.getLong("crtime"), rs.getLong("atime"), rs.getLong("mtime"),
@@ -156,7 +156,7 @@ class ResultSetHelper {
 				TSK_FS_ATTR_TYPE_ENUM.valueOf(rs.getShort("attr_type")),
 				rs.getShort("attr_id"), name, rs.getLong("meta_addr"),
 				TSK_FS_NAME_TYPE_ENUM.valueOf(rs.getShort("dir_type")),
-				TSK_FS_META_TYPE_ENUM.ValueOf(rs.getShort("meta_type")),
+				TSK_FS_META_TYPE_ENUM.valueOf(rs.getShort("meta_type")),
 				TSK_FS_NAME_FLAG_ENUM.valueOf(rs.getShort("dir_flags")),
 				rs.getShort("meta_flags"), rs.getLong("size"),
 				rs.getLong("ctime"), rs.getLong("crtime"), rs.getLong("atime"), rs.getLong("mtime"),
@@ -183,7 +183,7 @@ class ResultSetHelper {
 		final VirtualDirectory vd = new VirtualDirectory(db, rs.getLong("obj_id"),
 				rs.getString("name"),
 				TSK_FS_NAME_TYPE_ENUM.valueOf(rs.getShort("dir_type")),
-				TSK_FS_META_TYPE_ENUM.ValueOf(rs.getShort("meta_type")),
+				TSK_FS_META_TYPE_ENUM.valueOf(rs.getShort("meta_type")),
 				TSK_FS_NAME_FLAG_ENUM.valueOf(rs.getShort("dir_flags")), rs.getShort("meta_flags"),
 				rs.getLong("size"), rs.getString("md5"),
 				FileKnown.valueOf(rs.getByte("known")), parentPath);
@@ -240,7 +240,7 @@ class ResultSetHelper {
 		final DerivedFile df =
 				new DerivedFile(db, objId, rs.getString("name"),
 				TSK_FS_NAME_TYPE_ENUM.valueOf(rs.getShort("dir_type")),
-				TSK_FS_META_TYPE_ENUM.ValueOf(rs.getShort("meta_type")),
+				TSK_FS_META_TYPE_ENUM.valueOf(rs.getShort("meta_type")),
 				TSK_FS_NAME_FLAG_ENUM.valueOf(rs.getShort("dir_flags")), rs.getShort("meta_flags"),
 				rs.getLong("size"),
 				rs.getLong("ctime"), rs.getLong("crtime"), rs.getLong("atime"), rs.getLong("mtime"),
@@ -275,7 +275,7 @@ class ResultSetHelper {
 		final LocalFile lf =
 				new LocalFile(db, objId, rs.getString("name"),
 				TSK_FS_NAME_TYPE_ENUM.valueOf(rs.getShort("dir_type")),
-				TSK_FS_META_TYPE_ENUM.ValueOf(rs.getShort("meta_type")),
+				TSK_FS_META_TYPE_ENUM.valueOf(rs.getShort("meta_type")),
 				TSK_FS_NAME_FLAG_ENUM.valueOf(rs.getShort("dir_flags")), rs.getShort("meta_flags"),
 				rs.getLong("size"),
 				rs.getLong("ctime"), rs.getLong("crtime"), rs.getLong("atime"), rs.getLong("mtime"),
