@@ -103,6 +103,19 @@ public interface SleuthkitItemVisitor<T> {
 	 */
     T visit(VirtualDirectory ld);
 
+	/**
+	 * Act on (visit) a DerivedFile content object
+	 * @param df derived file to visit / act on
+	 * @return result of the visit
+	 */
+    T visit(DerivedFile df);
+	
+	/**
+	 * Act on (visit) a LocalFile content object
+	 * @param lf local file to visit / act on
+	 * @return result of the visit
+	 */
+    T visit(LocalFile lf);
 	
 	/**
 	 * The default visitor - quickest method for implementing a custom visitor.  
@@ -160,5 +173,20 @@ public interface SleuthkitItemVisitor<T> {
 		public T visit(LayoutFile lf) {
 			return defaultVisit(lf);
 		}
+		
+		@Override
+		public T visit(VirtualDirectory vd) {
+			return defaultVisit(vd);
+		}
+		
+		@Override
+		public T visit(DerivedFile df) {
+			return defaultVisit(df);
+		}
+		
+		@Override
+		public T visit(LocalFile lf) {
+			return defaultVisit(lf);
+		}
     }
 }
