@@ -52,6 +52,11 @@
 //#define mkdir _mkdir
 #endif
 
+
+#ifdef HAVE_SYS_RESOURCE_H
+#include <sys/resource.h>
+#endif
+
 /* Output Devices */
 class arff *a = 0;			// ARFF generator
 class xml  *x = 0;
@@ -659,13 +664,6 @@ int main(int argc, char * const *argv1)
 	fprintf(stderr,"ERROR: fiwalk was compiled without AFF support.\n");
 	exit(0);
 #else
-#if 0
-	if((tsk_img_type_supported() & TSK_IMG_TYPE_AFF_AFF)==0){
-	    fprintf(stderr,"ERROR: fiwalk was compiled with AFF support but the TSK library is not.\n");
-	    fprintf(stderr,"tsk_img_type_supported=0x%x\n",tsk_img_type_supported());
-	    exit(0);
-	}
-#endif
 #endif
     }
 
@@ -753,6 +751,7 @@ int main(int argc, char * const *argv1)
 
     comment("clock: %s",tvbuf);
 
+#ifdef HAVE_SYS_RESOURCE_H
 #ifdef HAVE_GETRUSAGE
     /* Print usage information */
     struct rusage ru;
@@ -771,6 +770,7 @@ int main(int argc, char * const *argv1)
 	comment("stop_time: %s",cstr(mytime()));
 	if(x) x->pop();
     }
+#endif
 #endif
 
     // *** Added <finished time="(time_t)" duration="<seconds>" />
