@@ -3,12 +3,10 @@
 ** ext2fs_journal
 ** The Sleuth Kit 
 **
-** $Date: 2007/12/20 20:32:38 $
-**
 ** Journaling code for TSK_FS_INFO_TYPE_EXT_3 image
 **
-** Brian Carrier [carrier@sleuthkit.org]
-** Copyright (c) 2006-2007 Brian Carrier, Basis Technology.  All Rights reserved
+** Brian Carrier [carrier <at> sleuthkit [dot] org]
+** Copyright (c) 2006-2011 Brian Carrier, Basis Technology.  All Rights reserved
 ** Copyright (c) 2004-2005 Brian Carrier.  All rights reserved 
 **
 **
@@ -17,7 +15,7 @@
 */
 
 /** \file ext2fs_journal.c
- * Ext3 journal walking code.
+ * Contains the internal TSK Ext3 journal walking code.
  */
 
 #include "tsk_fs_i.h"
@@ -38,18 +36,19 @@
  */
 
 static TSK_WALK_RET_ENUM
-load_sb_action(TSK_FS_INFO * fs, TSK_DADDR_T addr, char *buf, size_t size,
-    TSK_FS_BLOCK_FLAG_ENUM flags, void *ptr)
+load_sb_action(TSK_FS_FILE * fs_file, TSK_OFF_T a_off, TSK_DADDR_T addr,
+    char *buf, size_t size, TSK_FS_BLOCK_FLAG_ENUM flags, void *ptr)
 {
+    TSK_FS_INFO *fs = fs_file->fs_info;
     ext2fs_journ_sb *sb;
     EXT2FS_INFO *ext2fs = (EXT2FS_INFO *) fs;
     EXT2FS_JINFO *jinfo = ext2fs->jinfo;
 
     if (size < 1024) {
         tsk_error_reset();
-        tsk_errno = TSK_ERR_FS_FUNC;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "FS block size is less than 1024, not supported in journal yet");
+        tsk_error_set_errno(TSK_ERR_FS_UNSUPFUNC);
+        tsk_error_set_errstr
+            ("FS block size is less than 1024, not supported in journal yet");
         return TSK_WALK_ERROR;
     }
 
@@ -57,9 +56,8 @@ load_sb_action(TSK_FS_INFO * fs, TSK_DADDR_T addr, char *buf, size_t size,
 
     if (big_tsk_getu32(sb->magic) != EXT2_JMAGIC) {
         tsk_error_reset();
-        tsk_errno = TSK_ERR_FS_MAGIC;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "Journal inode %" PRIuINUM
+        tsk_error_set_errno(TSK_ERR_FS_MAGIC);
+        tsk_error_set_errstr("Journal inode %" PRIuINUM
             " does not have a valid magic value: %" PRIx32,
             jinfo->j_inum, big_tsk_getu32(sb->magic));
         return TSK_WALK_ERROR;
@@ -89,8 +87,8 @@ ext2fs_jopen(TSK_FS_INFO * fs, TSK_INUM_T inum)
 
     if (!fs) {
         tsk_error_reset();
-        tsk_errno = TSK_ERR_FS_ARG;
-        snprintf(tsk_errstr, TSK_ERRSTR_L, "ext2fs_jopen: fs is null");
+        tsk_error_set_errno(TSK_ERR_FS_ARG);
+        tsk_error_set_errstr("ext2fs_jopen: fs is null");
         return 1;
     }
 
@@ -101,19 +99,18 @@ ext2fs_jopen(TSK_FS_INFO * fs, TSK_INUM_T inum)
     }
     jinfo->j_inum = inum;
 
-    jinfo->fs_inode = fs->inode_lookup(fs, inum);
-    if (!jinfo->fs_inode) {
+    jinfo->fs_file = tsk_fs_file_open_meta(fs, NULL, inum);
+    if (!jinfo->fs_file) {
         free(jinfo);
         return 1;
 //      error("error finding journal inode %" PRIu32, inum);
     }
 
-    if (fs->file_walk(fs, jinfo->fs_inode, 0, 0, TSK_FS_FILE_FLAG_NOID,
-            load_sb_action, NULL)) {
+    if (tsk_fs_file_walk(jinfo->fs_file, 0, load_sb_action, NULL)) {
         tsk_error_reset();
-        tsk_errno = TSK_ERR_FS_FWALK;
-        snprintf(tsk_errstr, TSK_ERRSTR_L, "Error loading ext3 journal");
-        tsk_fs_inode_free(jinfo->fs_inode);
+        tsk_error_set_errno(TSK_ERR_FS_FWALK);
+        tsk_error_set_errstr("Error loading ext3 journal");
+        tsk_fs_file_close(jinfo->fs_file);
         free(jinfo);
         return 1;
     }
@@ -142,45 +139,48 @@ ext2fs_jentry_walk(TSK_FS_INFO * fs, int flags,
     TSK_FS_LOAD_FILE buf1;
     TSK_DADDR_T i;
     int b_desc_seen = 0;
+    ext2fs_journ_sb *journ_sb = NULL;
+    ext4fs_journ_commit_head *commit_head;
 
     // clean up any error messages that are lying around
     tsk_error_reset();
 
 
-    if (jinfo == NULL) {
+    if ((jinfo == NULL) || (jinfo->fs_file == NULL)
+        || (jinfo->fs_file->meta == NULL)) {
         tsk_error_reset();
-        tsk_errno = TSK_ERR_FS_ARG;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "ext2fs_jentry_walk: journal is not open");
+        tsk_error_set_errno(TSK_ERR_FS_ARG);
+        tsk_error_set_errstr("ext2fs_jentry_walk: journal is not open");
         return 1;
     }
 
-    if (jinfo->fs_inode->size != (jinfo->last_block + 1) * jinfo->bsize) {
+    if (jinfo->fs_file->meta->size !=
+        (jinfo->last_block + 1) * jinfo->bsize) {
         tsk_error_reset();
-        tsk_errno = TSK_ERR_FS_ARG;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "ext2fs_jentry_walk: journal file size is different from \nsize reported in journal super block");
+        tsk_error_set_errno(TSK_ERR_FS_ARG);
+        tsk_error_set_errstr
+            ("ext2fs_jentry_walk: journal file size is different from \nsize reported in journal super block");
         return 1;
     }
 
     /* Load the journal into a buffer */
-    buf1.left = buf1.total = (size_t) jinfo->fs_inode->size;
+    buf1.left = buf1.total = (size_t) jinfo->fs_file->meta->size;
     journ = buf1.cur = buf1.base = tsk_malloc(buf1.left);
     if (journ == NULL) {
         return 1;
     }
 
-    if (fs->file_walk(fs, jinfo->fs_inode, 0, 0, TSK_FS_FILE_FLAG_NOID,
-            tsk_fs_load_file_action, (void *) &buf1)) {
+    if (tsk_fs_file_walk(jinfo->fs_file,
+            0, tsk_fs_load_file_action, (void *) &buf1)) {
         free(journ);
         return 1;
     }
 
     if (buf1.left > 0) {
         tsk_error_reset();
-        tsk_errno = TSK_ERR_FS_FWALK;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "ext2fs_jentry_walk: Buffer not fully copied");
+        tsk_error_set_errno(TSK_ERR_FS_FWALK);
+        tsk_error_set_errstr
+            ("ext2fs_jentry_walk: Buffer not fully copied");
         free(journ);
         return 1;
     }
@@ -189,7 +189,7 @@ ext2fs_jentry_walk(TSK_FS_INFO * fs, int flags,
     /* Process the journal 
      * Cycle through each block
      */
-    tsk_printf("JBlk\tDescriptrion\n");
+    tsk_printf("JBlk\tDescription\n");
 
     /* Note that 'i' is incremented when we find a descriptor block and
      * process its contents. */
@@ -327,6 +327,29 @@ ext2fs_jentry_walk(TSK_FS_INFO * fs, int flags,
             (big_tsk_getu32(head->entry_type) == EXT2_J_ETYPE_SB2)) {
             tsk_printf("%" PRIuDADDR ":\tSuperblock (seq: %" PRIu32 ")\n",
                 i, big_tsk_getu32(head->entry_seq));
+            journ_sb = head;
+            tsk_printf("sb version: %d\n",
+                big_tsk_getu32(head->entry_type));
+            tsk_printf("sb version: %d\n",
+                big_tsk_getu32(journ_sb->entrytype));
+            tsk_printf("sb feature_compat flags 0x%08X\n",
+                big_tsk_getu32(journ_sb->feature_compat));
+            if (big_tsk_getu32(journ_sb->
+                    feature_compat) & JBD2_FEATURE_COMPAT_CHECKSUM)
+                tsk_printf("\tJOURNAL_CHECKSUMS\n");
+            tsk_printf("sb feature_incompat flags 0x%08X\n",
+                big_tsk_getu32(journ_sb->feature_incompat));
+            if (big_tsk_getu32(journ_sb->
+                    feature_incompat) & JBD2_FEATURE_INCOMPAT_REVOKE)
+                tsk_printf("\tJOURNAL_REVOKE\n");
+            if (big_tsk_getu32(journ_sb->
+                    feature_incompat) & JBD2_FEATURE_INCOMPAT_64BIT)
+                tsk_printf("\tJOURNAL_64BIT\n");
+            if (big_tsk_getu32(journ_sb->
+                    feature_incompat) & JBD2_FEATURE_INCOMPAT_ASYNC_COMMIT)
+                tsk_printf("\tJOURNAL_ASYNC_COMMIT\n");
+            tsk_printf("sb feature_ro_incompat flags 0x%08X\n",
+                big_tsk_getu32(journ_sb->feature_ro_incompat));
         }
 
         /* Revoke Block */
@@ -340,11 +363,44 @@ ext2fs_jentry_walk(TSK_FS_INFO * fs, int flags,
 
         /* The commit is the end of the entries */
         else if (big_tsk_getu32(head->entry_type) == EXT2_J_ETYPE_COM) {
-            tsk_printf("%" PRIuDADDR ":\t%sCommit Block (seq: %" PRIu32
-                ")\n", i, ((i < jinfo->start_blk)
+            tsk_printf("%" PRIuDADDR ":\t%sCommit Block (seq: %" PRIu32, i,
+                ((i < jinfo->start_blk)
                     || (big_tsk_getu32(head->entry_seq) <
                         jinfo->start_seq)) ? "Unallocated " : "Allocated ",
                 big_tsk_getu32(head->entry_seq));
+            commit_head = head;
+            //tsk_printf("commit seq %" PRIu32 "\n", big_tsk_getu32(commit_head->c_header.entry_seq));
+            if (big_tsk_getu32(journ_sb->
+                    feature_compat) & JBD2_FEATURE_COMPAT_CHECKSUM) {
+                int chksum_type = commit_head->chksum_type;
+                if (chksum_type) {
+                    tsk_printf(", checksum_type: %d",
+                        commit_head->chksum_type);
+                    switch (commit_head->chksum_type) {
+                    case JBD2_CRC32_CHKSUM:
+                        tsk_printf("-CRC32");
+                        break;
+                    case JBD2_MD5_CHKSUM:
+                        tsk_printf("-MD5");
+                        break;
+                    case JBD2_SHA1_CHKSUM:
+                        tsk_printf("-SHA1");
+                        break;
+                    default:
+                        tsk_printf("-UNKOWN");
+                        break;
+                    }
+                    tsk_printf(", checksum_size: %d",
+                        commit_head->chksum_size);
+                    tsk_printf(", chksum: 0x%08X",
+                        big_tsk_getu32(commit_head->chksum));
+                }
+            }
+            tsk_printf(", sec: %llu.%u", tsk_getu64(TSK_BIG_ENDIAN,
+                    commit_head->commit_sec),
+                NSEC_PER_SEC / 10 * tsk_getu32(TSK_BIG_ENDIAN,
+                    commit_head->commit_nsec));
+            tsk_printf(")\n");
         }
 
         /* The descriptor describes the FS blocks that follow it */
@@ -400,8 +456,8 @@ ext2fs_jentry_walk(TSK_FS_INFO * fs, int flags,
                     break;
 
                 /* If the SAMEID value is set, then we advance by the size of the entry, otherwise add 16 for the ID */
-                else if (big_tsk_getu32(dentry->
-                        flag) & EXT2_J_DENTRY_SAMEID)
+                else if (big_tsk_getu32(dentry->flag) &
+                    EXT2_J_DENTRY_SAMEID)
                     dentry =
                         (ext2fs_journ_dentry *) ((uintptr_t) dentry +
                         sizeof(ext2fs_journ_dentry));
@@ -428,8 +484,8 @@ ext2fs_jentry_walk(TSK_FS_INFO * fs, int flags,
  * Return 0 on success and 1 on error
  */
 uint8_t
-ext2fs_jblk_walk(TSK_FS_INFO * fs, TSK_DADDR_T start, TSK_DADDR_T end, int flags,
-    TSK_FS_JBLK_WALK_CB action, void *ptr)
+ext2fs_jblk_walk(TSK_FS_INFO * fs, TSK_DADDR_T start, TSK_DADDR_T end,
+    int flags, TSK_FS_JBLK_WALK_CB action, void *ptr)
 {
     EXT2FS_INFO *ext2fs = (EXT2FS_INFO *) fs;
     EXT2FS_JINFO *jinfo = ext2fs->jinfo;
@@ -441,35 +497,35 @@ ext2fs_jblk_walk(TSK_FS_INFO * fs, TSK_DADDR_T start, TSK_DADDR_T end, int flags
     // clean up any error messages that are lying around
     tsk_error_reset();
 
-    if (jinfo == NULL) {
+    if ((jinfo == NULL) || (jinfo->fs_file == NULL)
+        || (jinfo->fs_file->meta == NULL)) {
         tsk_error_reset();
-        tsk_errno = TSK_ERR_FS_ARG;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "ext2fs_jblk_walk: journal is not open");
+        tsk_error_set_errno(TSK_ERR_FS_ARG);
+        tsk_error_set_errstr("ext2fs_jblk_walk: journal is not open");
         return 1;
     }
 
     if (jinfo->last_block < end) {
         tsk_error_reset();
-        tsk_errno = TSK_ERR_FS_WALK_RNG;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "ext2fs_jblk_walk: end is too large ");
+        tsk_error_set_errno(TSK_ERR_FS_WALK_RNG);
+        tsk_error_set_errstr("ext2fs_jblk_walk: end is too large ");
         return 1;
     }
 
     if (start != end) {
         tsk_error_reset();
-        tsk_errno = TSK_ERR_FS_ARG;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "ext2fs_blk_walk: only start == end is currently supported");
+        tsk_error_set_errno(TSK_ERR_FS_ARG);
+        tsk_error_set_errstr
+            ("ext2fs_blk_walk: only start == end is currently supported");
         return 1;
     }
 
-    if (jinfo->fs_inode->size != (jinfo->last_block + 1) * jinfo->bsize) {
+    if (jinfo->fs_file->meta->size !=
+        (jinfo->last_block + 1) * jinfo->bsize) {
         tsk_error_reset();
-        tsk_errno = TSK_ERR_FS_FUNC;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "ext2fs_jblk_walk: journal file size is different from size reported in journal super block");
+        tsk_error_set_errno(TSK_ERR_FS_UNSUPFUNC);
+        tsk_error_set_errstr
+            ("ext2fs_jblk_walk: journal file size is different from size reported in journal super block");
         return 1;
     }
 
@@ -483,17 +539,16 @@ ext2fs_jblk_walk(TSK_FS_INFO * fs, TSK_DADDR_T start, TSK_DADDR_T end, int flags
         return 1;
     }
 
-    if (fs->file_walk(fs, jinfo->fs_inode, 0, 0, TSK_FS_FILE_FLAG_NOID,
-            tsk_fs_load_file_action, (void *) &buf1)) {
+    if (tsk_fs_file_walk(jinfo->fs_file, 0, tsk_fs_load_file_action,
+            (void *) &buf1)) {
         free(journ);
         return 1;
     }
 
     if (buf1.left > 0) {
         tsk_error_reset();
-        tsk_errno = TSK_ERR_FS_FWALK;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "ext2fs_jblk_walk: Buffer not fully copied");
+        tsk_error_set_errno(TSK_ERR_FS_FWALK);
+        tsk_error_set_errstr("ext2fs_jblk_walk: Buffer not fully copied");
         free(journ);
         return 1;
     }
@@ -571,9 +626,9 @@ ext2fs_jblk_walk(TSK_FS_INFO * fs, TSK_DADDR_T start, TSK_DADDR_T end, int flags
 
     if (fwrite(&journ[end * jinfo->bsize], jinfo->bsize, 1, stdout) != 1) {
         tsk_error_reset();
-        tsk_errno = TSK_ERR_FS_WRITE;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "ext2fs_jblk_walk: error writing buffer block");
+        tsk_error_set_errno(TSK_ERR_FS_WRITE);
+        tsk_error_set_errstr
+            ("ext2fs_jblk_walk: error writing buffer block");
         free(journ);
         return 1;
     }
