@@ -1,14 +1,11 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2007/12/20 20:32:38 $
- *
- * Brian Carrier [carrier@sleuthkit.org]
- * Copyright (c) 2003-2007 Brian Carrier.  All rights reserved
+ * Brian Carrier [carrier <at> sleuthkit [dot] org]
+ * Copyright (c) 2003-2011 Brian Carrier.  All rights reserved
  *
  *
  * This software is distributed under the Common Public License 1.0
- *
  */
 
 #include "tsk_hashdb_i.h"
@@ -55,8 +52,8 @@ lCode"
         return TSK_HDB_NSRL_FORM2;
 
     tsk_error_reset();
-    tsk_errno = TSK_ERR_HDB_CORRUPT;
-    snprintf(tsk_errstr, TSK_ERRSTR_L,
+    tsk_error_set_errno(TSK_ERR_HDB_CORRUPT);
+    tsk_error_set_errstr(
              "nsrl: Unknown header format: %s\n", str);
     return -1;
 }
@@ -92,6 +89,17 @@ nsrl_test(FILE * hFile)
     return 1;
 }
 
+/**
+ * Set db_name using information from this database type
+ *
+ * @param hdb_info the hash database object
+ */
+void
+nsrl_name(TSK_HDB_INFO * hdb_info)
+{
+    tsk_hdb_name_from_path(hdb_info);
+}
+
 /**
  * Perform a basic check on a string to see if it starts with quotes
  * and contains a possible SHA-1 value
@@ -124,8 +132,8 @@ nsrl_parse_sha1(char *str, char **sha1, char **name, int ver)
     /* Sanity check */
     if (is_valid_nsrl(str) == 0) {
         tsk_error_reset();
-        tsk_errno = TSK_ERR_HDB_CORRUPT;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
+        tsk_error_set_errno(TSK_ERR_HDB_CORRUPT);
+        tsk_error_set_errstr(
                  "nsrl_parse_sha1: Invalid string to parse: %s", str);
         return 1;
     }
@@ -139,8 +147,8 @@ nsrl_parse_sha1(char *str, char **sha1, char **name, int ver)
         /* Final sanity check to make sure there are no ',' in hash */
         if (NULL != strchr(ptr, ',')) {
             tsk_error_reset();
-            tsk_errno = TSK_ERR_HDB_CORRUPT;
-            snprintf(tsk_errstr, TSK_ERRSTR_L,
+            tsk_error_set_errno(TSK_ERR_HDB_CORRUPT);
+            tsk_error_set_errstr(
                      "nsrl_parse_sha1: Invalid string to parse (commas after SHA1): %s",
                      ptr);
             return 1;
@@ -159,8 +167,8 @@ nsrl_parse_sha1(char *str, char **sha1, char **name, int ver)
 
             if (NULL == (ptr = strchr(ptr, ','))) {
                 tsk_error_reset();
-                tsk_errno = TSK_ERR_HDB_CORRUPT;
-                snprintf(tsk_errstr, TSK_ERRSTR_L,
+                tsk_error_set_errno(TSK_ERR_HDB_CORRUPT);
+                tsk_error_set_errstr(
                          "nsrl_parse_sha1: Invalid string to parse (commas after name): %s",
                          ptr);
                 return 1;
@@ -179,8 +187,8 @@ nsrl_parse_sha1(char *str, char **sha1, char **name, int ver)
 
             if (NULL == (ptr = strchr(ptr, ','))) {
                 tsk_error_reset();
-                tsk_errno = TSK_ERR_HDB_CORRUPT;
-                snprintf(tsk_errstr, TSK_ERRSTR_L,
+                tsk_error_set_errno(TSK_ERR_HDB_CORRUPT);
+                tsk_error_set_errstr(
                          "nsrl_parse_sha1: Invalid string to parse (commas after name): %s",
                          ptr);
                 return 1;
@@ -214,8 +222,8 @@ nsrl_parse_md5(char *str, char **md5, char **name, int ver)
     /* Sanity check */
     if (is_valid_nsrl(str) == 0) {
         tsk_error_reset();
-        tsk_errno = TSK_ERR_HDB_CORRUPT;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
+        tsk_error_set_errno(TSK_ERR_HDB_CORRUPT);
+        tsk_error_set_errstr(
                  "nsrl_parse_md5: Invalid string to parse: %s", str);
         return 1;
     }
@@ -249,8 +257,8 @@ nsrl_parse_md5(char *str, char **md5, char **name, int ver)
             else if ((cnt == 2) && (name != NULL)) {
                 if (ptr[-1] != '"') {
                     tsk_error_reset();
-                    tsk_errno = TSK_ERR_HDB_CORRUPT;
-                    snprintf(tsk_errstr, TSK_ERRSTR_L,
+                    tsk_error_set_errno(TSK_ERR_HDB_CORRUPT);
+                    tsk_error_set_errstr(
                              "nsrl_parse_md5: Missing Quote after name: %s",
                              (char *) name);
                     return 1;
@@ -268,8 +276,8 @@ nsrl_parse_md5(char *str, char **md5, char **name, int ver)
                     || (ptr[1] != '"')
                     || (ptr[2 + TSK_HDB_HTYPE_MD5_LEN] != '"')) {
                     tsk_error_reset();
-                    tsk_errno = TSK_ERR_HDB_CORRUPT;
-                    snprintf(tsk_errstr, TSK_ERRSTR_L,
+                    tsk_error_set_errno(TSK_ERR_HDB_CORRUPT);
+                    tsk_error_set_errstr(
                              "nsrl_parse_md5: Invalid MD5 value: %s", ptr);
                     return 1;
                 }
@@ -282,8 +290,8 @@ nsrl_parse_md5(char *str, char **md5, char **name, int ver)
                 /* Final sanity check */
                 if (NULL != strchr(ptr, ',')) {
                     tsk_error_reset();
-                    tsk_errno = TSK_ERR_HDB_CORRUPT;
-                    snprintf(tsk_errstr, TSK_ERRSTR_L,
+                    tsk_error_set_errno(TSK_ERR_HDB_CORRUPT);
+                    tsk_error_set_errstr(
                              "nsrl_parse_md5: Missing comma after MD5: %s",
                              (char *) md5);
                     return 1;
@@ -298,8 +306,8 @@ nsrl_parse_md5(char *str, char **md5, char **name, int ver)
             if (ptr[1] == '"') {
                 if (NULL == (ptr = strchr(&ptr[2], '"'))) {
                     tsk_error_reset();
-                    tsk_errno = TSK_ERR_HDB_CORRUPT;
-                    snprintf(tsk_errstr, TSK_ERRSTR_L,
+                    tsk_error_set_errno(TSK_ERR_HDB_CORRUPT);
+                    tsk_error_set_errstr(
                              "nsrl_parse_md5: Error advancing past quote");
                     return 1;
                 }
@@ -319,8 +327,8 @@ nsrl_parse_md5(char *str, char **md5, char **name, int ver)
             /* Final sanity check to make sure there are no ',' in hash */
             if (NULL != strchr(ptr, ',')) {
                 tsk_error_reset();
-                tsk_errno = TSK_ERR_HDB_CORRUPT;
-                snprintf(tsk_errstr, TSK_ERRSTR_L,
+                tsk_error_set_errno(TSK_ERR_HDB_CORRUPT);
+                tsk_error_set_errstr(
                          "nsrl_parse_md5: Comma in MD5 value: %s", ptr);
                 return 1;
             }
@@ -338,8 +346,8 @@ nsrl_parse_md5(char *str, char **md5, char **name, int ver)
 
             if (NULL == (ptr = strchr(ptr, ','))) {
                 tsk_error_reset();
-                tsk_errno = TSK_ERR_HDB_CORRUPT;
-                snprintf(tsk_errstr, TSK_ERRSTR_L,
+                tsk_error_set_errno(TSK_ERR_HDB_CORRUPT);
+                tsk_error_set_errstr(
                          "nsrl_parse_md5: Missing comma after name: %s",
                          (char *) name);
                 return 1;
@@ -352,8 +360,8 @@ nsrl_parse_md5(char *str, char **md5, char **name, int ver)
         return 0;
     }
     tsk_error_reset();
-    tsk_errno = TSK_ERR_HDB_ARG;
-    snprintf(tsk_errstr, TSK_ERRSTR_L,
+    tsk_error_set_errno(TSK_ERR_HDB_ARG);
+    tsk_error_set_errstr(
              "nsrl_parse_md5: Invalid version: %d\n", ver);
     return 1;
 }
@@ -382,7 +390,7 @@ nsrl_makeindex(TSK_HDB_INFO * hdb_info, TSK_TCHAR * dbtype)
     int db_cnt = 0, idx_cnt = 0, ig_cnt = 0;
 
     if (tsk_hdb_idxinitialize(hdb_info, dbtype)) {
-        snprintf(tsk_errstr2, TSK_ERRSTR_L, "nsrl_makeindex");
+        tsk_error_set_errstr2( "nsrl_makeindex");
         return 1;
     }
 
@@ -433,7 +441,7 @@ nsrl_makeindex(TSK_HDB_INFO * hdb_info, TSK_TCHAR * dbtype)
 
         /* Add the entry to the index */
         if (tsk_hdb_idxaddentry(hdb_info, hash, offset)) {
-            snprintf(tsk_errstr2, TSK_ERRSTR_L, "nsrl_makeindex");
+            tsk_error_set_errstr2( "nsrl_makeindex");
             return 1;
         }
 
@@ -455,14 +463,14 @@ nsrl_makeindex(TSK_HDB_INFO * hdb_info, TSK_TCHAR * dbtype)
 
         /* Close and sort the index */
         if (tsk_hdb_idxfinalize(hdb_info)) {
-            snprintf(tsk_errstr2, TSK_ERRSTR_L, "nsrl_makeindex");
+            tsk_error_set_errstr2( "nsrl_makeindex");
             return 1;
         }
     }
     else {
         tsk_error_reset();
-        tsk_errno = TSK_ERR_HDB_CORRUPT;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
+        tsk_error_set_errno(TSK_ERR_HDB_CORRUPT);
+        tsk_error_set_errstr(
                  "nsrl_makeindex: No valid entries found in database");
         return 1;
     }
@@ -505,8 +513,8 @@ nsrl_getentry(TSK_HDB_INFO * hdb_info, const char *hash, TSK_OFF_T offset,
     if ((hdb_info->hash_type == TSK_HDB_HTYPE_MD5_ID)
         && (strlen(hash) != TSK_HDB_HTYPE_MD5_LEN)) {
         tsk_error_reset();
-        tsk_errno = TSK_ERR_HDB_ARG;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
+        tsk_error_set_errno(TSK_ERR_HDB_ARG);
+        tsk_error_set_errstr(
                  "nsrl_getentry: Invalid hash value (expected to be MD5): %s\n",
                  hash);
         return 1;
@@ -514,8 +522,8 @@ nsrl_getentry(TSK_HDB_INFO * hdb_info, const char *hash, TSK_OFF_T offset,
     else if ((hdb_info->hash_type == TSK_HDB_HTYPE_SHA1_ID)
              && (strlen(hash) != TSK_HDB_HTYPE_SHA1_LEN)) {
         tsk_error_reset();
-        tsk_errno = TSK_ERR_HDB_ARG;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
+        tsk_error_set_errno(TSK_ERR_HDB_ARG);
+        tsk_error_set_errstr(
                  "nsrl_getentry: Invalid hash value (expected to be SHA1): %s\n",
                  hash);
         return 1;
@@ -525,14 +533,14 @@ nsrl_getentry(TSK_HDB_INFO * hdb_info, const char *hash, TSK_OFF_T offset,
     fseeko(hdb_info->hDb, 0, SEEK_SET);
     if (NULL == fgets(buf, TSK_HDB_MAXLEN, hdb_info->hDb)) {
         tsk_error_reset();
-        tsk_errno = TSK_ERR_HDB_READDB;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
+        tsk_error_set_errno(TSK_ERR_HDB_READDB);
+        tsk_error_set_errstr(
                  "nsrl_getentry: Error reading NSRLFile.txt header\n");
         return 1;
     }
 
     if ((ver = get_format_ver(buf)) == -1) {
-        snprintf(tsk_errstr2, TSK_ERRSTR_L, "nsrl_getentry");
+        tsk_error_set_errstr2( "nsrl_getentry");
         return 1;
     }
 
@@ -544,8 +552,8 @@ nsrl_getentry(TSK_HDB_INFO * hdb_info, const char *hash, TSK_OFF_T offset,
 
         if (0 != fseeko(hdb_info->hDb, offset, SEEK_SET)) {
             tsk_error_reset();
-            tsk_errno = TSK_ERR_HDB_READDB;
-            snprintf(tsk_errstr, TSK_ERRSTR_L,
+            tsk_error_set_errno(TSK_ERR_HDB_READDB);
+            tsk_error_set_errstr(
                      "nsrl_getentry: Error seeking to get file name: %lu",
                      (unsigned long) offset);
             return 1;
@@ -556,8 +564,8 @@ nsrl_getentry(TSK_HDB_INFO * hdb_info, const char *hash, TSK_OFF_T offset,
                 break;
 
             tsk_error_reset();
-            tsk_errno = TSK_ERR_HDB_READDB;
-            snprintf(tsk_errstr, TSK_ERRSTR_L,
+            tsk_error_set_errno(TSK_ERR_HDB_READDB);
+            tsk_error_set_errstr(
                      "nsrl_getentry: Error reading database");
             return 1;
         }
@@ -565,8 +573,8 @@ nsrl_getentry(TSK_HDB_INFO * hdb_info, const char *hash, TSK_OFF_T offset,
         len = strlen(buf);
         if (len < TSK_HDB_HTYPE_SHA1_LEN + 5) {
             tsk_error_reset();
-            tsk_errno = TSK_ERR_HDB_CORRUPT;
-            snprintf(tsk_errstr, TSK_ERRSTR_L,
+            tsk_error_set_errno(TSK_ERR_HDB_CORRUPT);
+            tsk_error_set_errstr(
                      "nsrl_getentry: Invalid entry in database (too short): %s",
                      buf);
             return 1;
@@ -576,8 +584,8 @@ nsrl_getentry(TSK_HDB_INFO * hdb_info, const char *hash, TSK_OFF_T offset,
         if (hdb_info->hash_type == TSK_HDB_HTYPE_SHA1_ID) {
             if (nsrl_parse_sha1(buf, &cur_hash, &name, ver)) {
                 tsk_error_reset();
-                tsk_errno = TSK_ERR_HDB_CORRUPT;
-                snprintf(tsk_errstr, TSK_ERRSTR_L,
+                tsk_error_set_errno(TSK_ERR_HDB_CORRUPT);
+                tsk_error_set_errstr(
                          "nsrl_getentry: Invalid entry in database: %s",
                          buf);
                 return 1;
@@ -586,8 +594,8 @@ nsrl_getentry(TSK_HDB_INFO * hdb_info, const char *hash, TSK_OFF_T offset,
         else if (hdb_info->hash_type == TSK_HDB_HTYPE_MD5_ID) {
             if (nsrl_parse_md5(buf, &cur_hash, &name, ver)) {
                 tsk_error_reset();
-                tsk_errno = TSK_ERR_HDB_CORRUPT;
-                snprintf(tsk_errstr, TSK_ERRSTR_L,
+                tsk_error_set_errno(TSK_ERR_HDB_CORRUPT);
+                tsk_error_set_errstr(
                          "nsrl_getentry: Invalid entry in database: %s",
                          buf);
                 return 1;
@@ -618,8 +626,8 @@ nsrl_getentry(TSK_HDB_INFO * hdb_info, const char *hash, TSK_OFF_T offset,
 
     if (found == 0) {
         tsk_error_reset();
-        tsk_errno = TSK_ERR_HDB_ARG;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
+        tsk_error_set_errno(TSK_ERR_HDB_ARG);
+        tsk_error_set_errstr(
                  "nsrl_getentry: Hash not found in file at offset: %lu",
                  (unsigned long) offset);
         return 1;
